! ============================================================================ !
!   scanner.h    A (class) proximity scanner
! ============================================================================ !
!   Version    :  v1.0 dicembre 2004
!
!   Autors     :  Alessandro Schillaci & Paolo Lucchesi
!
!   Description:  A class implementing a scanner: with this extension
!                 it's possible to list all instances of a class in the rooms
!                 contiguous to the current location without really moving
!
!   License    :  Scanner.h is free software; you can redistribute it and/or
!                 modify it under the terms of the GNU General Public License
!                 as published by the Free Software Foundation; either version
!                 2 of the License, or (at your option) any later version.
!
!                 scanner.h is distributed in the hope that it will be useful,
!                 but WITHOUT ANY WARRANTY; without even the implied warranty
!                 of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
!                 See the GNU General Public License for more details.
!
!                 You should have received a copy of the GNU General Public
!                 License along with this program; if not, write to the Free
!                 Software Foundation, Inc., 59 Temple Place - Suite 330,
!                 Boston, MA  02111-1307, USA.
! ============================================================================ !
!
!     Usage    :
!
!     1- Add the following line
!           Include "scanner";
!        in your source file, AFTER including "parser"
!
!     2- Add a new verb, like this
!
!        Verb 'scanner'   				*        -> 	Scan; 
!
!        [ ScanSub;
!	          print  "You turn on the scanner.......scanning......";
!	          searchnear(NPC);
!        ];
!
!     3- Objects that should be detected by the scannes should have a
!        "remotedesc" properties. Otherwise, their standard description
!        will be printed
!
! ============================================================================ !
! NOTES:    One can customize the scanner verb, so that it will react and
!           search different kinds of objects (as an example, a scanner
!           sensible to people and inanimate object, or a scanner sensible
!           to particular classes of objects can be useful.)
!           It's useful if the player wants to know what's inside a room
!           WITHOUT entering it.
! ============================================================================ !
message "Compiling scanner.h (v1.0)";

[Searchroom clss rm x;
    objectloop (x in rm) {
        if (x ofclass clss) {
            if (x provides remotedesc) x.remotedesc();
            else if (x provides description) x.description();
            else print "You can see ", (a) x, ".^";
        }
    }
];

[Searchnear clazz room i dest desttype;
    ! Found the current location
    room = location;
    while (parent(room)) room = parent(room);

    ! Explore the contiguous rooms
    objectloop (i in compass) {
        dest = room.(i.door_dir);
        if (dest) {
            desttype = ZRegion(dest);
            if ((desttype ~= 3) &&
                (desttype == 2 || dest hasnt door || dest hasnt concealed)) {
                print "^";
                PrintDirectionName(i);
                print " (", (name) dest,")^";

                ! Search current room for clazz class
                Searchroom (clazz,dest);
            }
        }
    }

];

[ PrintDirectionName obj;
    switch (obj) {
	        n_obj:   print "to the north";
	        s_obj:   print "to the south";
	        e_obj:   print "to the east";
	        w_obj:   print "to the west";
	        ne_obj:  print "to the north-east";
	        se_obj:  print "to the south-east";
	        nw_obj:  print "to the north-west";
	        sw_obj:  print "to the south-west";
	        u_obj:   print "up";
	        d_obj:   print "down";
	        in_obj:  print "inside";
	        out_obj: print "outside";
	        default: print "direction not specified";
        }
];


